-- ---------------------------------------------------------------------------------
-- 
-- Converts the ivy system database from version 62 to version 63
-- 
-- ---------------------------------------------------------------------------------
-- 
-- This script was automatically generated. Do not edit it. Instead edit the source file
-- 
-- ---------------------------------------------------------------------------------
-- Database: Microsoft SQL Server
-- ---------------------------------------------------------------------------------
-- Copyright:
-- Axon Ivy AG, Baarerstrasse 12, 6300 Zug
-- ---------------------------------------------------------------------------------

-- Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_TaskQuery
GO

DROP VIEW IWA_CaseQuery
GO

-- Store Procedure to drop a unique constraint
CREATE PROCEDURE IWA_Drop_ForeignKey
@tableName varchar(100),
@columnName varchar(100)
AS
DECLARE @fkName Varchar(255)
SET @fkName= (
  SELECT fk.name
  FROM sys.foreign_keys fk
  INNER JOIN sys.foreign_key_columns fkc ON fkc.constraint_object_id = fk.object_id
  INNER JOIN sys.columns c1 ON fkc.parent_column_id = c1.column_id AND fkc.parent_object_id = c1.object_id
  INNER JOIN sys.columns c2 ON fkc.referenced_column_id = c2.column_id AND fkc.referenced_object_id = c2.object_id
  WHERE OBJECT_NAME(fk.parent_object_id)=@tableName AND c2.name=@columnName
)
DECLARE @dropSql varchar(4000)
SET @dropSql=
  'ALTER TABLE [dbo].['+@tableName+']
    DROP CONSTRAINT ['+@fkName+']'
EXEC(@dropSql)

GO

EXECUTE IWA_Drop_ForeignKey @tableName='IWA_Environment', @columnName='BusinessCalendarId'
GO
EXECUTE IWA_Drop_ForeignKey @tableName='IWA_Case', @columnName='BusinessCalendarId'
GO
EXECUTE IWA_Drop_ForeignKey @tableName='IWA_Task', @columnName='BusinessCalendarId'
GO

-- Drop trigger of no longer exisiting table
DROP TRIGGER IWA_BusinessCalendarDeleteTrigger
GO

-- Drop no longer exisiting table
DROP TABLE IWA_BusinessCalendar
GO

-- Drop unique constraint which depend on changed columns or was deleted
-- Store Procedure to drop a unique constraint
CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_Application'
GO

-- Dropped columns of table IWA_Application
ALTER TABLE IWA_Application DROP COLUMN BusinessCalendarId
GO


-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_Application ADD UNIQUE (Name)
GO

-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_Environment'
GO

-- Dropped columns of table IWA_Environment
ALTER TABLE IWA_Environment DROP COLUMN BusinessCalendarId
GO


-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_Environment ADD UNIQUE (ApplicationId, Name)
GO

-- Dropped columns of table IWA_Case
ALTER TABLE IWA_Case DROP COLUMN BusinessCalendarId
GO


-- Dropped columns of table IWA_Task
ALTER TABLE IWA_Task DROP COLUMN BusinessCalendarId
GO


-- Recreate views which has a new definition or depend on changed tables
CREATE VIEW IWA_TaskQuery
(
  TaskId,
  CaseId,
  BusinessCaseId,
  ProcessModelId,
  ApplicationId,
  StartTaskSwitchEventId,
  EndTaskSwitchEventId,
  TaskStartId,
  TaskEndId,
  WorkerUserId,
  WorkerUserName,
  WorkerUserDisplayName,
  WorkerSessionId,
  ActivatorRoleId,
  ActivatorUserId,
  ActivatorName,
  ActivatorDisplayName,
  ExpiryActivatorRoleId,
  ExpiryActivatorUserId,
  ExpiryActivatorName,
  ExpiryActivatorDisplayName,
  ExpiryPriority,
  ExpiryTimestamp,
  ExpiryTaskStartElementPid,
  IsExpired,
  ExpiredCreatorTaskId,
  TimeoutedCreatorIntrmdtEventId,
  DelayTimestamp,
  "State",
  RequestPath,
  DisplayNameTemplate,
  Name,
  DisplayDescriptionTemplate,
  Description,
  Priority,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  FailedTimeoutTimestamp,
  NumberOfFailures,
  NumberOfResumes,
  Category,
  IsUpdatedOnStart,
  IsOffline,
  CurrentPriority,
  CurrentActivatorName,
  CurrentActivatorDisplayName,
  CurrentActivatorRoleId,
  CurrentActivatorUserId
)
AS
  SELECT
    IWA_Task.TaskId,
    IWA_Task.CaseId,
    IWA_Task.BusinessCaseId,
    IWA_Task.ProcessModelId,
    IWA_Task.ApplicationId,
    IWA_Task.StartTaskSwitchEventId,
    IWA_Task.EndTaskSwitchEventId,
    IWA_Task.TaskStartId,
    IWA_Task.TaskEndId,
    IWA_Task.WorkerUserId,
    IWA_Task.WorkerUserName,
    CASE WHEN WorkerUserId IS NOT NULL AND LEN(WorkerUser.FullName) > 0 THEN WorkerUser.FullName WHEN WorkerUserId IS NOT NULL THEN WorkerUser.Name ELSE NULL END,
    IWA_Task.WorkerSessionId,
    IWA_Task.ActivatorRoleId,
    IWA_Task.ActivatorUserId,
    IWA_Task.ActivatorName,
    CASE WHEN IWA_Task.ActivatorUserId IS NOT NULL AND LEN(ActivatorUser.FullName) > 0 THEN ActivatorUser.FullName WHEN IWA_Task.ActivatorUserId IS NOT NULL THEN ActivatorUser.Name WHEN IWA_Task.ActivatorRoleId IS NOT NULL AND LEN(ActivatorRole.DisplayNameTemplate) > 0 THEN ActivatorRole.DisplayNameTemplate WHEN IWA_Task.ActivatorRoleId IS NOT NULL THEN ActivatorRole.Name ELSE NULL END,
    IWA_Task.ExpiryActivatorRoleId,
    IWA_Task.ExpiryActivatorUserId,
    IWA_Task.ExpiryActivatorName,
    CASE WHEN IWA_Task.ExpiryActivatorUserId IS NOT NULL AND LEN(ExpiryActivatorUser.FullName) > 0 THEN ExpiryActivatorUser.FullName WHEN IWA_Task.ExpiryActivatorUserId IS NOT NULL THEN ExpiryActivatorUser.Name WHEN IWA_Task.ExpiryActivatorRoleId IS NOT NULL AND LEN(ExpiryActivatorRole.DisplayNameTemplate) > 0 THEN ExpiryActivatorRole.DisplayNameTemplate WHEN IWA_Task.ExpiryActivatorRoleId IS NOT NULL THEN ExpiryActivatorRole.Name ELSE NULL END,
    IWA_Task.ExpiryPriority,
    IWA_Task.ExpiryTimestamp,
    IWA_Task.ExpiryTaskStartElementPid,
    IWA_Task.IsExpired,
    IWA_Task.ExpiredCreatorTaskId,
    IWA_Task.TimeoutedCreatorIntrmdtEventId,
    IWA_Task.DelayTimestamp,
    IWA_Task."State",
    IWA_Task.RequestPath,
    IWA_Task.DisplayNameTemplate,
    IWA_Task.Name,
    IWA_Task.DisplayDescriptionTemplate,
    IWA_Task.Description,
    IWA_Task.Priority,
    IWA_Task.StartTimestamp,
    IWA_Task.EndTimestamp,
    IWA_Task.BusinessCalendar,
    IWA_Task.WorkingTime,
    IWA_Task.BusinessRuntime,
    IWA_Task.FailedTimeoutTimestamp,
    IWA_Task.NumberOfFailures,
    IWA_Task.NumberOfResumes,
    IWA_Task.Category,
    IWA_Task.IsUpdatedOnStart,
    IWA_Task.IsOffline,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryPriority WHEN 0 THEN IWA_Task.Priority END,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryActivatorName WHEN 0 THEN IWA_Task.ActivatorName END,
    CASE WHEN IWA_Task.IsExpired = 1 AND IWA_Task.ExpiryActivatorUserId IS NOT NULL AND LEN(ExpiryActivatorUser.FullName) > 0 THEN ExpiryActivatorUser.FullName WHEN IWA_Task.IsExpired = 1 AND IWA_Task.ExpiryActivatorUserId IS NOT NULL THEN ExpiryActivatorUser.Name WHEN IWA_Task.IsExpired = 1 AND IWA_Task.ExpiryActivatorRoleId IS NOT NULL AND LEN(ExpiryActivatorRole.DisplayNameTemplate) > 0 THEN ExpiryActivatorRole.DisplayNameTemplate WHEN IWA_Task.IsExpired = 1 AND IWA_Task.ExpiryActivatorRoleId IS NOT NULL THEN ExpiryActivatorRole.Name WHEN IWA_Task.IsExpired = 0 AND IWA_Task.ActivatorUserId IS NOT NULL AND LEN(ActivatorUser.FullName) > 0 THEN ActivatorUser.FullName WHEN IWA_Task.IsExpired = 0 AND IWA_Task.ActivatorUserId IS NOT NULL THEN ActivatorUser.Name WHEN IWA_Task.IsExpired = 0 AND IWA_Task.ActivatorRoleId IS NOT NULL AND LEN(ActivatorRole.DisplayNameTemplate) > 0 THEN ActivatorRole.DisplayNameTemplate WHEN IWA_Task.IsExpired = 0 AND IWA_Task.ActivatorRoleId IS NOT NULL THEN ActivatorRole.Name ELSE NULL END,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryActivatorRoleId WHEN 0 THEN IWA_Task.ActivatorRoleId END,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryActivatorUserId WHEN 0 THEN IWA_Task.ActivatorUserId END
  FROM IWA_Task
    LEFT OUTER JOIN IWA_User AS ActivatorUser ON IWA_Task.ActivatorUserId = ActivatorUser.UserId
    LEFT OUTER JOIN IWA_Role AS ActivatorRole ON IWA_Task.ActivatorRoleId = ActivatorRole.RoleId
    LEFT OUTER JOIN IWA_User AS ExpiryActivatorUser ON IWA_Task.ExpiryActivatorUserId = ExpiryActivatorUser.UserId
    LEFT OUTER JOIN IWA_Role AS ExpiryActivatorRole ON IWA_Task.ExpiryActivatorRoleId = ExpiryActivatorRole.RoleId
    LEFT OUTER JOIN IWA_User AS WorkerUser ON IWA_Task.WorkerUserId = WorkerUser.UserId
GO

CREATE VIEW IWA_CaseQuery
(
  CaseId,
  BusinessCaseId,
  Kind,
  ApplicationId,
  ProcessModelId,
  TaskStartId,
  CreatorUserId,
  CreatorUserName,
  CreatorUserDisplayName,
  CreatorTaskId,
  EnvironmentId,
  DisplayNameTemplate,
  Name,
  DisplayDescriptionTemplate,
  Description,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  "State",
  Priority,
  Stage,
  OwnerRoleId,
  OwnerUserId,
  OwnerName,
  Category
)
AS
  SELECT
    IWA_Case.CaseId,
    IWA_Case.BusinessCaseId,
    IWA_Case.Kind,
    IWA_Case.ApplicationId,
    IWA_Case.ProcessModelId,
    IWA_Case.TaskStartId,
    IWA_Case.CreatorUserId,
    IWA_Case.CreatorUserName,
    CASE WHEN IWA_Case.CreatorUserId IS NOT NULL AND LEN(IWA_User.FullName) > 0 THEN IWA_User.FullName WHEN IWA_Case.CreatorUserId IS NOT NULL THEN IWA_User.Name ELSE NULL END,
    IWA_Case.CreatorTaskId,
    IWA_Case.EnvironmentId,
    IWA_Case.DisplayNameTemplate,
    IWA_Case.Name,
    IWA_Case.DisplayDescriptionTemplate,
    IWA_Case.Description,
    IWA_Case.StartTimestamp,
    IWA_Case.EndTimestamp,
    IWA_Case.BusinessCalendar,
    IWA_Case.WorkingTime,
    IWA_Case.BusinessRuntime,
    IWA_Case."State",
    IWA_Case.Priority,
    IWA_Case.Stage,
    IWA_Case.OwnerRoleId,
    IWA_Case.OwnerUserId,
    IWA_Case.OwnerName,
    IWA_Case.Category
  FROM IWA_Case
    LEFT OUTER JOIN IWA_User ON IWA_Case.CreatorUserId = IWA_User.UserId
GO


-- Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_ForeignKey
GO
DROP PROCEDURE IWA_Drop_Unique
GO

-- 
-- Update Version
-- 
UPDATE IWA_Version SET Version=63
GO
